package kernel.log;

import haxe.ds.ReadOnlyArray;
#if webconsole
import lib.Debug;
#end

using tink.CoreApi;

/**
	Central logging system.
**/
class Log {
	private static inline final MAX_LINES:Int = 100;

	public static var onLog(default, null):Signal<LogLine>;

	private static final onLogTrigger:SignalTrigger<LogLine> = new SignalTrigger();
	private static final logLines:Array<LogLine> = [];

	@:allow(kernel.Init)
	private static function init() {
		onLog = onLogTrigger.asSignal();

		#if debug
		haxe.Log.trace = function(v:Dynamic, ?infos:haxe.PosInfos) {
			Log.debug(v, infos);
		}
		#end
	}

	public static function info(msg:Dynamic, ?pos:haxe.PosInfos) {
		log({level: Info, message: Std.string(msg), time: 0}, pos);
	}

	public static function warn(msg:Dynamic, ?pos:haxe.PosInfos) {
		log({level: Warn, message: Std.string(msg), time: 0}, pos);
	}

	public static function error(msg:Dynamic, ?pos:haxe.PosInfos) {
		log({level: Error, message: Std.string(msg), time: 0}, pos);
	}

	public static function debug(msg:Dynamic, ?pos:haxe.PosInfos) {
		#if debug
		log({level: Debug, message: Std.string(msg), time: 0}, pos);
		#end
	}

	public static function silly(msg:Dynamic, ?pos:haxe.PosInfos) {
		log({level: Silly, message: Std.string(msg), time: 0}, pos);
	}

	private static function log(line:LogLine, ?pos:haxe.PosInfos) {
		line.origin = pos.className;
		logLines.push(line);

		if (logLines.length > MAX_LINES) {
			logLines.shift();
		}
		onLogTrigger.trigger(line);

		#if webconsole
		Debug.printWeb('[${Std.string(line.level)}][${line.origin}] ${line.message}');
		#end
	}

	public static function getLines():ReadOnlyArray<LogLine> {
		return logLines;
	}
}
