package macros.rpc;

import haxe.macro.TypeTools;
import haxe.macro.Context;
import haxe.macro.Expr;

class RPC {
	macro static public function buildRPC(iface:Expr):Array<Field> {
		var buildFields = Context.getBuildFields();
		var localClass = Context.getLocalClass().get();
		var handleExprs:Array<Expr> = [];

		var ifaceType = null;

		switch (iface.expr) {
			case EConst(CIdent(i)):
				var t = Context.getType(i);
				ifaceType = TypeTools.toComplexType(t);
				switch (t) {
					case TInst(t2, params):
						var fields = t2.get().fields.get();
						for (field in fields) {
							switch (field.type) {
								case TFun(args, ret):
									var argsExprs:Array<Expr> = [for (a in args) macro $i{a.name}];

									var convertedArgs:Array<FunctionArg> = [];
									for (a in args) {
										convertedArgs.push({
											name: a.name,
											opt: a.opt,
											type: TypeTools.toComplexType(a.t)
										});
									}

									var retComplexType = TypeTools.toComplexType(ret);
									var retType:ComplexType = if (Helper.isPromise(ret)) {
										TypeTools.toComplexType(Helper.getPromiseType(retComplexType));
									} else {
										retComplexType;
									}

									buildFields.push({
										name: field.name,
										pos: localClass.pos,
										access: [APublic, AInline],
										kind: FFun({
											args: convertedArgs,
											expr: macro {
												return cast this._performRequest($v{field.name}, $a{argsExprs});
											},
											ret: Helper.newPromise(retType),
										}),
									});

									// Add expr for package handle
									// TODO: this needs to be more efficient. Maybe use a switch case.

									var callArgs = [for (k => v in args) macro p.data.args[$v{k}]];
									var funName = field.name;

									if (Helper.isVoid(ret)) {
										handleExprs.push(macro {
											if (p.data.func == $v{funName}) {
												d.$funName($a{callArgs});
												p.respond(null);
												return true;
											}

											true;
										});
									} else if (Helper.isPromise(ret)) {
										handleExprs.push(macro {
											if (p.data.func == $v{funName}) {
												d.$funName($a{callArgs}).handle((r) -> {
													p.respond(r);
													return true;
												});
											}

											true;
										});
									} else {
										handleExprs.push(macro {
											if (p.data.func == $v{funName}) {
												p.respond(d.$funName($a{callArgs}));
												return true;
											}

											// HACK: I not a 100% sure why this need to be here but it does not work without it.
											// My guess is that a macro has to be able to be evaluated to a value and a simple if
											// statement is of type Void. I don't think this last true statement will have an impact
											// on the resulting Expr.
											true;
										});
									}

								default:
									Context.warning("IDFK man", field.pos);
							}
						}
					default:
						Context.error("Only Interfaces are supported", iface.pos);
				}

			default:
				Context.error("Only Interfaces are supported", iface.pos);
		}

		// Add handle package static method
		buildFields.push({
			name: "handlePackage",
			access: [APublic, AStatic],
			pos: localClass.pos,
			kind: FFun({
				args: [
					{
						name: "d",
						type: ifaceType,
						opt: false,
					},
					{
						name: "p",
						type: macro :kernel.net.Package.GenericPackage,
						opt: false,
					},
				],
				expr: macro {
					$a{handleExprs} return false;
				}
			})
		});

		return buildFields;
	}
}
